"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = require("crypto");
const redis_1 = require("redis");
const debug_1 = __importDefault(require("debug"));
const ALGORITHM = "aes-256-cbc", IV_LENGTH = 16;
const log = (0, debug_1.default)("secure-store-redis");
/**
 * SecureStore class
 *
 * Automatically encrypt any data saved to redis
 *
 * @export
 * @class SecureStore
 */
class SecureStore {
    /**
     * Creates an instance of SecureStore.
     *
     * @constructor
     */
    constructor(cfg) {
        if (typeof cfg.redis !== "object") {
            cfg.redis = {};
        }
        if (typeof cfg.uid !== "undefined") {
            if (typeof cfg.uid !== "string") {
                throw new Error("If specifying a UID, it must be a string");
            }
        }
        else {
            cfg.uid = (0, crypto_1.randomBytes)(4).toString("hex");
        }
        if (typeof cfg.secret !== "undefined") {
            if (typeof cfg.secret !== "string" || cfg.secret.length !== 32) {
                throw new Error(`If specifying a secret, it must be a 32 char string (length: ${cfg.secret.length})`);
            }
        }
        else {
            cfg.secret = (0, crypto_1.randomBytes)(16).toString("hex");
        }
        this.config = cfg;
    }
    /**
     * Disconnects the Redis client
     */
    disconnect() {
        return __awaiter(this, arguments, void 0, function* (client = this.client) {
            if (client) {
                log("Redis client quit called");
                yield client.quit();
                try {
                    log("Redis client disconnect called");
                    yield client.disconnect();
                }
                catch (e) {
                    log("Redis disconnect failed, ignoring");
                }
            }
        });
    }
    /**
     * Initializes (connects) the Redis client to the Redis server
     */
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.client) {
                return new Promise((resolve, reject) => {
                    let error = false;
                    const client = (0, redis_1.createClient)(this.config.redis);
                    client.on("error", (err) => __awaiter(this, void 0, void 0, function* () {
                        error = true;
                        log("Redis connection error", err);
                        this.disconnect(client);
                        return reject(err);
                    }));
                    client.connect().then(() => {
                        if (!error) {
                            log("Connected to Redis");
                            this.client = client;
                            resolve();
                        }
                    });
                });
            }
        });
    }
    /**
     * Save and encrypt arbitrary data to Redis
     */
    save(key_1, data_1) {
        return __awaiter(this, arguments, void 0, function* (key, data, postfix = "") {
            if (typeof key !== "string") {
                throw new Error("No hash key specified");
            }
            else if (!data) {
                throw new Error("No data provided, nothing to save");
            }
            postfix = postfix ? ":" + postfix : "";
            if (typeof data === "object") {
                try {
                    data = JSON.stringify(data);
                }
                catch (e) {
                    throw new Error(e);
                }
            }
            yield this.init();
            data = this.encrypt(data);
            const hash = SecureStore.shasum(key);
            return this.client.HSET(this.config.uid + postfix, hash, data);
        });
    }
    /**
     * Get and decrypt arbitrary data from Redis
     */
    get(key_1) {
        return __awaiter(this, arguments, void 0, function* (key, postfix = "") {
            if (typeof key !== "string") {
                throw new Error("No hash key specified");
            }
            postfix = postfix ? ":" + postfix : "";
            yield this.init();
            const hash = SecureStore.shasum(key);
            const res = yield this.client.HGET(this.config.uid + postfix, hash);
            let data;
            if (typeof res === "string") {
                try {
                    data = this.decrypt(res);
                }
                catch (e) {
                    log("Failed to decrypt data");
                    return null;
                }
                try {
                    data = JSON.parse(data);
                }
                catch (e) {
                    log("Failed to parse dataset as JSON");
                }
            }
            else {
                data = res;
            }
            return data;
        });
    }
    /**
     * Delete arbitrary data from Redis
     */
    delete(key_1) {
        return __awaiter(this, arguments, void 0, function* (key, postfix = "") {
            if (typeof key !== "string") {
                throw new Error("No hash key specified");
            }
            postfix = postfix ? ":" + postfix : "";
            yield this.init();
            const hash = SecureStore.shasum(key);
            return this.client.HDEL(this.config.uid + postfix, hash);
        });
    }
    /**
     * Encrypts arbitrary data, returning an encrypted string
     */
    encrypt(data) {
        const iv = (0, crypto_1.randomBytes)(IV_LENGTH);
        const cipher = (0, crypto_1.createCipheriv)(ALGORITHM, Buffer.from(this.config.secret), iv);
        let encrypted = cipher.update(data);
        encrypted = Buffer.concat([encrypted, cipher.final()]);
        return iv.toString("hex") + ":" + encrypted.toString("hex");
    }
    /**
     * Decrypts given encrypted string, returning its arbitrary data
     */
    decrypt(encrypted) {
        const parts = encrypted.split(":");
        const iv = Buffer.from(parts.shift(), "hex");
        const encryptedText = Buffer.from(parts.join(":"), "hex");
        const decipher = (0, crypto_1.createDecipheriv)(ALGORITHM, Buffer.from(this.config.secret), iv);
        let decrypted = decipher.update(encryptedText);
        decrypted = Buffer.concat([decrypted, decipher.final()]);
        return decrypted.toString();
    }
    /**
     * Generate sha256 sum from given text
     */
    static shasum(text) {
        const s = (0, crypto_1.createHash)("sha256");
        s.update(text);
        return s.digest("hex");
    }
}
exports.default = SecureStore;
//# sourceMappingURL=/index.js.map